<?php
/*
Plugin Name: LeagueManager
Plugin URI: http://wordpress.org/extend/plugins/leaguemanager/
Description: Manage and present sports league results.
Version: 4.2-RC1.3.2
Author: Kolja, LaMonte Forthun

Copyright 2008-2019  Kolja
					 LaMonte Forthun (email : lamontef@collegefundsoftware.com, lamontef@yahoo.com)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

 /**
  * Main class to implement LeagueManager
  *
  */
class LeagueManager {
	/**
	 * Plugin version
	 *
	 * @var string
	 */
	private $version = '4.2-RC1.3.2';


	/**
	 * Database version
	 *
	 * @var string
	 */
	private $dbversion = '3.8.6';


	/**
	 * reporting message
	 *
	 * @var string
	 */
	private $message = '';
	
	/**
	 * error variable
	 *
	 * @var boolean
	 */
	private $error = false;

	
	/**
	 * constructor
	 *
	 */
	public function __construct() {
		global $lmStats, $wpdb;
		
		$wpdb->show_errors();
		$this->loadOptions();
		$this->defineConstants();
		$this->defineTables();
		$this->loadTextdomain();
		$this->loadLibraries();

		register_activation_hook(__FILE__, array(&$this, 'activate') );

		if (function_exists('register_uninstall_hook'))
			register_uninstall_hook(__FILE__, array('LeagueManager', 'uninstall'));

		add_action( 'widgets_init', array(&$this, 'registerWidget') );
		
		add_action('wp_enqueue_scripts', array(&$this, 'loadStyles'), 5 );
		add_action('wp_enqueue_scripts', array(&$this, 'loadScripts') );
		
		// Add TinyMCE Button
		add_action( 'init', array(&$this, 'addTinyMCEButton') );
		
		// register AJAX action to show TinyMCE Window
		add_action( 'wp_ajax_leaguemanager_tinymce_window', array(&$this, 'showTinyMCEWindow') );

		// Enable matches in Fancy slideshows
		add_filter( 'fancy_slideshow_sources', array( &$this, 'addSlideshowSources' ) );
		add_filter( 'fancy_slideshow_overlay_styles', array( &$this, 'addSlideshowOverlayStyles' ) );
		add_filter( 'fancy_slideshow_get_slides_matches', array(&$this, 'getSlideshowSlides') );
		
		// activate ProjectManager extensions
		if ( $this->hasProjectManager() ) {
			require_once (dirname (__FILE__) . '/lib/projectmanager.php');
			$ext = new LeagueManagerProjectManager();
		}
	}
	
	
	/**
	 * Add individual league matches to FancySlideshows categories
	 *
	 * @param array $categories
	 * @return array
	 */
	public function addSlideshowSources( $categories ) {
		$categories['leagues_matches'] = array( "title" => __( 'League Matches', 'leaguemanager'), "options" => array() );
		foreach ( $this->getLeagues() AS $league ) {
			$categories["leagues_matches"]["options"][] = array( "value" => "matches_".$league->id, "label" => $league->title );
			// Include Options for next and previous matches
			$times = array( 'next' => __('Next Matches', 'leaguemanager'), 'prev' => __('Previous Matches', 'leaguemanager') );
			foreach ( $times AS $time => $name ) {
				$categories["leagues_matches"]["options"][] = array( "value" => sprintf("matches_%d_%s", $league->id, $time), "label" => sprintf("%s - %s", $league->title, $name) );
			}
		}
		
		return $categories;
	}
	
	
	/**
	 * add custom league overlay style to FancySlideshows
	 *
	 * @param array $styles
	 * @return array
	 */
	public function addSlideshowOverlayStyles( $styles ) {
		$styles['leaguemanager'] = __( 'LeagueManager', 'leaguemanager' );
		return $styles;
	}
	
	
	/**
	 * retrieve FancySlideshows slides
	 *
	 * @param array $category
	 * @return array
	 */
	public function getSlideshowSlides( $category ) {
		$league = get_league( $category[1] );
		
		if ( $category[0] == 'matches' ) {
			$show_logos = ( $league->slideshow['show_logos'] == 1 ) ? true : false;
			$time = isset($category[2]) ? $category[2] : false;
			$limit = intval($league->slideshow['num_matches']);
			//$latest_season = end($league->seasons);
			if ($league->slideshow['season'] == 'latest') {
				$season = end($league->seasons);
				if (!empty($season))
					$season = $season['name'];
			} else {
				$season = $league->slideshow['season'];
			}
			$matches = $league->getMatches( array('season' => $season, 'match_day' => -1, 'time' => $time, 'limit' => $limit) );
			$team_args = array("season" => false, "orderby" => array("id" => "ASC"));
			
			$slides = array();
			if ( $matches ) {
				foreach ( $matches AS $match ) {
					$title = $match->getTitle(false);
					
					if ( $time == 'next' ) $match->score = ' v.s. ';
					$score = "<span class='result'>".$match->score."</span>";
					
					$homeTeam = $match->teams['home'];
					$awayTeam = $match->teams['away'];
					
					$slide_title = "";
					$slide_title .= "<p class='match_title'><strong>".$title."</strong></p>";
					if ( $show_logos )
						$slide_title .= "<p class='logos'><img class='logo home_logo' src='".$homeTeam->getImageURL($homeTeam->logo, 'thumb')."' alt='' />".$score."<img class='logo away_logo' src='".$awayTeam->getImageURL($awayTeam->logo, 'thumb')."' alt='' /></p>";
					else
						$slide_title .= "<p class='logos'>".$score."</p>";

					$slide_desc = "";
					if ( !empty($match->match_day) )
						$slide_desc .= "<p class='match_day'>".sprintf(__("<strong>%d.</strong> Match Day", 'leaguemanager'), $match->match_day)."</p>";
					
					$slide_desc .= "<p class='date'>".$match->match_date.", <span class='time'>".$match->start_time."</span></p>";
					$slide_desc .= "<p class='location'>".$match->location."</p>";
				
					if ( $match->report != "" )
						$slide_desc .=  "<p class='report'>".$match->report."</p>";
				
					$slide = array(
						"name" => $title,
						"title" => $title,
						"slide_title" => $slide_title,
						"slide_desc" => $slide_desc
					);
					
					$slides[] = (object)$slide;
				}
			}
			
			return $slides;
		}
		
		return array();
	}


	/**
	 * register Widget
	 */
	public function registerWidget() {
		register_widget('LeagueManagerWidget');
	}


	/**
	 * define constants
	 *
	 */
	private function defineConstants() {
		if (!defined('LEAGUEMANAGER_VERSION')) {
			/**
			 * Plugin version
			 *
			 * @var string
			 */
			define( 'LEAGUEMANAGER_VERSION', $this->version );
		}
		
		if (!defined('LEAGUEMANAGER_DBVERSION')) {
			/**
			 * Database version
			 *
			 * @var string
			 */
			define( 'LEAGUEMANAGER_DBVERSION', $this->dbversion );
		}
		
		if (!defined('LEAGUEMANAGER_URL')) {
			/**
			 * Plugin URL
			 *
			 * @var string
			 */
			define( 'LEAGUEMANAGER_URL', rtrim(esc_url(plugin_dir_url(__FILE__)), "/") ); // remove trailing slash as the plugin has been coded without it
		}
	
		if (!defined('LEAGUEMANAGER_PATH')) {
			/**
			 * Plugin path
			 *
			 * @var string
			 */
			define( 'LEAGUEMANAGER_PATH', dirname(__FILE__) );
		}
	}


	/**
	 * define database tables
	 *
	 */
	private function defineTables() {
		global $wpdb;
		$wpdb->leaguemanager = $wpdb->prefix . 'leaguemanager_leagues';
		$wpdb->leaguemanager_teams = $wpdb->prefix . 'leaguemanager_teams';
		$wpdb->leaguemanager_matches = $wpdb->prefix . 'leaguemanager_matches';
		$wpdb->leaguemanager_stats = $wpdb->prefix . 'leaguemanager_stats';
	}


	/**
	 * load libraries
	 */
	private function loadLibraries() {
		// Objects
		require_once (dirname (__FILE__) . '/lib/league.php');
		require_once (dirname (__FILE__) . '/lib/team.php');
		require_once (dirname (__FILE__) . '/lib/match.php');
		require_once (dirname (__FILE__) . '/lib/championship.php');
		
		/*
		 * load sports libraries
		 */
		// First read files in leaguemanager sports directory, then overwrite with sports files in user stylesheet directory
		$files = array_merge($this->readDirectory(LEAGUEMANAGER_PATH."/sports"), $this->readDirectory(get_stylesheet_directory() . "/sports"));
		
		// load files
		foreach ( $files AS $file ) {
			require_once($file);
		}

		
		// Global libraries
		require_once (dirname (__FILE__) . '/lib/shortcodes.php');
		require_once (dirname (__FILE__) . '/lib/widget.php');
		require_once (dirname (__FILE__) . '/lib/ajax.php');
		
		
		//require_once (dirname (__FILE__) . '/lib/tiebreakers.php');
		
		// template tags & functions
		require_once(dirname(__FILE__) . '/template-tags.php');
		require_once (dirname (__FILE__) . '/functions.php');
		
		// register AJAX methods
		$leaguemanager_ajax = new LeagueManagerAJAX();
		// register Shortcodes
		$leaguemanager_shortcodes = new LeagueManagerShortcodes();
	}

	
	/**
	 * get standings display options
	 *
	 * @return array
	 */
	public function getStandingsDisplayOptions() {
		$options = array(
			'status' => __( 'Team Status', 'leaguemanager' ),
			'logos' => __( 'Logos', 'leaguemanager' ),
			'team_link' => __( 'Include Link to team page', 'leaguemanager' ),
			'pld' => __( 'Played Games', 'leaguemanager' ),
			'won' => __( 'Won Games', 'leaguemanager' ),
			'tie' => __('Tie Games', 'leaguemanager' ),
			'lost' => __( 'Lost Games', 'leaguemanager' ),
			'winPercent' => __( 'Win Percentage', 'leaguemanager' ),
			'last5' => __( 'Last 5 Matches', 'leaguemanager' )
		);
		
		/**
		 * Fires when standings options are generated
		 *
		 * @param array $options
		 * @return array
		 * @category wp-filter
		 */
		$options = apply_filters('league_standings_options', $options);
		
		return $options;
	}
	
	
	/**
	 * check if ProjectManager is present for extensions, including team roster & profile and match statistics
	 *
	 * @return boolean
	 */
	public function hasProjectManager() {
		include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
		
		if ( file_exists(WP_PLUGIN_DIR . '/projectmanager/projectmanager.php') && is_plugin_active("projectmanager/projectmanager.php") ) {
			$p = get_option('projectmanager');
			if (version_compare($p['version'], '3.6.1', '>=')) {
				return true;
			}
		}
		
		return false;
	}
	

	/**
	 * read files in directory
	 *
	 * @param string $dir
	 * @return array
	 */
	public function readDirectory($dir) {
		$files = array();
		
		if ( file_exists($dir) ) {
			if ( $handle = opendir($dir) ) {
				while ( false !== ($file = readdir($handle)) ) {
					$file_info = pathinfo($dir.'/'.$file);
					$file_type = (isset($file_info['extension'])) ? $file_info['extension'] : '';
					if ( $file != "." && $file != ".." && !is_dir($file) && substr($file, 0,1) != "."  && $file_type == 'php' )  {
						$files[$file] = $dir.'/'.$file;
					}
				}
			}
		}
		
		return $files;
	}
	
	
	/**
	 * load options
	 *
	 */
	private function loadOptions() {
		$this->options = get_option('leaguemanager');
	}


	/**
	 * get options
	 *
	 * @param string|false $index
	 */
	public function getOptions($index = false) {
		if ( $index )
			return $this->options[$index];
		
		return $this->options;
	}


	/**
	 * load textdomain
	 *
	 */
	private function loadTextdomain() {
		$textdomain = $this->getOptions('textdomain');
		if ( !empty($textdomain) ) {
			$locale = get_locale();
			$path = dirname(__FILE__) . '/languages';
			$domain = 'leaguemanager';
			$mofile = $path . '/'. $domain . '-' . $textdomain . '-' . $locale . '.mo';

			if ( file_exists($mofile) ) {
				load_textdomain($domain, $mofile);
				return true;
			}
		}
		
		load_plugin_textdomain( 'leaguemanager', false, 'leaguemanager/languages' );
	}


	/**
	 * load Javascript
	 *
	 */
	public function loadScripts() {
		wp_register_script( 'leaguemanager', LEAGUEMANAGER_URL.'/leaguemanager.js', array('jquery', 'jquery-ui-core', 'jquery-ui-accordion', 'jquery-ui-tabs', 'jquery-effects-core', 'jquery-effects-slide', 'sack', 'thickbox'), LEAGUEMANAGER_VERSION );
		wp_enqueue_script('leaguemanager');
		?>
		<script type="text/javascript">
		//<![CDATA[
		LeagueManagerAjaxL10n = {
			blogUrl: "<?php bloginfo( 'wpurl' ); ?>",
			//pluginPath: "<?php echo LEAGUEMANAGER_PATH; ?>",
			pluginUrl: "<?php echo LEAGUEMANAGER_URL; ?>",
			requestUrl: "<?php echo admin_url( 'admin-ajax.php' ) ?>",
			Edit: "<?php _e("Edit"); ?>",
			Post: "<?php _e("Post"); ?>",
			Save: "<?php _e("Save"); ?>",
			Cancel: "<?php _e("Cancel"); ?>",
			pleaseWait: "<?php _e("Please wait..."); ?>",
			Revisions: "<?php _e("Page Revisions"); ?>",
			Time: "<?php _e("Insert time"); ?>",
			Options: "<?php _e("Options") ?>",
			Delete: "<?php _e('Delete') ?>"
	 	}
		//]]>
		</script>
		<?php
	}


	/**
	 * load CSS styles
	 *
	 */
	public function loadStyles() {
		wp_enqueue_style('thickbox');
		wp_enqueue_style('leaguemanager', LEAGUEMANAGER_URL . "/style.css", false, '1.0', 'all');
		
		wp_register_style('jquery-ui', LEAGUEMANAGER_URL . "/css/jquery/jquery-ui.min.css", false, '1.11.4', 'all');
		wp_register_style('jquery-ui-structure', LEAGUEMANAGER_URL . "/css/jquery/jquery-ui.structure.min.css", array('jquery-ui'), '1.11.4', 'all');
		wp_register_style('jquery-ui-theme', LEAGUEMANAGER_URL . "/css/jquery/jquery-ui.theme.min.css", array('jquery-ui', 'jquery-ui-structure'), '1.11.4', 'all');
		
		wp_enqueue_style('jquery-ui-structure');
		wp_enqueue_style('jquery-ui-theme');
		
		//wp_register_style('jquery_ui_css', 'http://ajax.googleapis.com/ajax/libs/jqueryui/1.9.2/themes/smoothness/jquery-ui.css', false, '1.0', 'screen');
		//wp_enqueue_style('jquery_ui_css');

		ob_start();
		require_once(LEAGUEMANAGER_PATH.'/css/colors.css.php');
		$css = ob_get_contents();
		ob_end_clean();
		
		wp_add_inline_style( 'leaguemanager', $css );
	}


	/**
	 * add TinyMCE Button
	 */
	public function addTinyMCEButton() {
		// Don't bother doing this stuff if the current user lacks permissions
		if ( !current_user_can('edit_posts') && !current_user_can('edit_pages') ) return;

		// Check for LeagueManager capability
		if ( !current_user_can('manage_leaguemanager') ) return;

		// Add only in Rich Editor mode
		if ( get_user_option('rich_editing') == 'true') {
			add_filter("mce_external_plugins", array(&$this, 'addTinyMCEPlugin'));
			add_filter('mce_buttons', array(&$this, 'registerTinyMCEButton'));
		}
	}
	/**
	 * add TinyMCE Plugin
	 *
	 * @param array $plugin_array An array of TinyMCE plugins
	 * @param array
	 */
	public function addTinyMCEPlugin( $plugin_array ) {
		$plugin_array['LeagueManager'] = LEAGUEMANAGER_URL.'/admin/tinymce/editor_plugin.js';
		return $plugin_array;
	}
	/**
	 * register TinyMCE Button
	 *
	 * @param array $buttons An array of TinyMCE Buttons
	 * @return array
	 */
	public function registerTinyMCEButton( $buttons ) {
		array_push($buttons, "separator", "LeagueManager");
		return $buttons;
	}


	/**
	 * Display the TinyMCE Window.
	 *
	 */
	public function showTinyMCEWindow() {
		require_once( LEAGUEMANAGER_PATH . '/admin/tinymce/window.php' );
		exit;
	}
	
	
	/**
	 * get upload directory
	 *
	 * @param string|false $file
	 * @return string upload path
	 */
	public function getFilePath( $file = false ) {
		$base = WP_CONTENT_DIR.'/uploads/leagues';
			
		if ( $file ) {
			return $base .'/'. basename($file);
		} else {
			return $base;
		}
	}
	
	
	/**
	 * Activate plugin
	 */
	public function activate() {
		$options = array();
		$options['version'] = $this->version;
		$options['dbversion'] = $this->dbversion;
		$options['textdomain'] = 'default';
		$options['colors']['headers'] = '#dddddd';
		$options['colors']['rows'] = array( 'main' => '#ffffff', 'alternate' => '#efefef', 'ascend' => '#ffffff', 'descend' => '#ffffff', 'relegate' => '#ffffff');
		$options['dashboard_widget']['num_items'] = 4;
		$options['dashboard_widget']['show_author'] = 1;
		$options['dashboard_widget']['show_date'] = 1;
		$options['dashboard_widget']['show_summary'] = 1;
 
		add_option( 'leaguemanager', $options, '', 'yes' );
		add_option( 'leaguemanager_widget', array(), '', 'yes' );

		// create directory
		wp_mkdir_p($this->getFilePath());
		
		/*
		* Set Capabilities
		*/
		$role = get_role('administrator');
		if ( $role !== null ) {
			$role->add_cap('view_leagues');
			$role->add_cap('leaguemanager_settings');
			$role->add_cap('edit_leagues');
			$role->add_cap('edit_league_settings');
			$role->add_cap('del_leagues');
			$role->add_cap('edit_seasons');
			$role->add_cap('del_seasons');
			$role->add_cap('edit_teams');
			$role->add_cap('del_teams');
			$role->add_cap('edit_matches');
			$role->add_cap('del_matches');
			$role->add_cap('update_results');
			$role->add_cap('export_leagues');
			$role->add_cap('import_leagues');
			$role->add_cap('manage_leaguemanager');
			
			// old rules
			$role->add_cap('leaguemanager');
			$role->add_cap('league_manager'); 
		}
		
		$role = get_role('editor');
		if ( $role !== null ) {
			$role->add_cap('view_leagues');
			
			// old rules
			$role->add_cap('leaguemanager');
		}

		$this->install();
	}


	/**
	 * Install plugin
	 */
	public function install() {
		global $wpdb;
		include_once( ABSPATH.'/wp-admin/includes/upgrade.php' );

		$charset_collate = '';
		if ( $wpdb->has_cap( 'collation' ) ) {
			if ( ! empty($wpdb->charset) )
				$charset_collate = "DEFAULT CHARACTER SET $wpdb->charset";
			if ( ! empty($wpdb->collate) )
				$charset_collate .= " COLLATE $wpdb->collate";
		}

		$create_leagues_sql = "CREATE TABLE {$wpdb->leaguemanager} (
						`id` int( 11 ) NOT NULL AUTO_INCREMENT,
						`title` varchar( 100 ) NOT NULL default '',
						`settings` longtext NOT NULL,
						`seasons` longtext NOT NULL,
						PRIMARY KEY ( `id` )) $charset_collate;";
		maybe_create_table( $wpdb->leaguemanager, $create_leagues_sql );

		$create_teams_sql = "CREATE TABLE {$wpdb->leaguemanager_teams} (
						`id` int( 11 ) NOT NULL AUTO_INCREMENT ,
						`status` varchar( 50 ) NOT NULL default '&#8226;',
						`title` varchar( 100 ) NOT NULL default '',
						`logo` varchar( 150 ) NOT NULL default '',
						`website` varchar( 255 ) NOT NULL default '',
						`coach` varchar( 100 ) NOT NULL default '',
						`stadium` varchar( 150 ) NOT NULL default '',
						`home` tinyint( 1 ) NOT NULL default '0',
						`points_plus` float NOT NULL default '0',
						`points_minus` float NOT NULL default '0',
						`points2_plus` int( 11 ) NOT NULL default '0',
						`points2_minus` int( 11 ) NOT NULL default '0',
						`add_points` int( 11 ) NOT NULL default '0',
						`done_matches` int( 11 ) NOT NULL default '0',
						`won_matches` int( 11 ) NOT NULL default '0',
						`draw_matches` int( 11 ) NOT NULL default '0',
						`lost_matches` int( 11 ) NOT NULL default '0',
						`diff` int( 11 ) NOT NULL default '0',
						`group` varchar( 30 ) NOT NULL default '',
						`league_id` int( 11 ) NOT NULL,
						`season` varchar( 255 ) NOT NULL default '',
						`rank` int( 11 ) NOT NULL default '0',
						`roster` int( 11 ) NOT NULL default '0',
						`profile` int( 11 ) NOT NULL default '0',
						`custom` longtext NOT NULL,
						PRIMARY KEY ( `id` )) $charset_collate;";
		maybe_create_table( $wpdb->leaguemanager_teams, $create_teams_sql );

		$create_matches_sql = "CREATE TABLE {$wpdb->leaguemanager_matches} (
						`id` int( 11 ) NOT NULL AUTO_INCREMENT ,
						`group` varchar( 30 ) NOT NULL default '',
						`date` datetime NOT NULL default '0000-00-00',
						`home_team` varchar( 255 ) NOT NULL default '0',
						`away_team` varchar( 255 ) NOT NULL default '0',
						`match_day` tinyint( 4 ) NOT NULL default '0',
						`location` varchar( 100 ) NOT NULL default '',
						`status` varchar( 255 ) NOT NULL default '',
						`league_id` int( 11 ) NOT NULL default '0',
						`season` varchar( 255 ) NOT NULL default '',
						`home_points` varchar( 30 ) NULL default NULL,
						`away_points` varchar( 30 ) NULL default NULL,
						`winner_id` int( 11 ) NOT NULL default '0',
						`loser_id` int( 11 ) NOT NULL default '0',
						`stat_id` int( 11 ) NOT NULL default '0',
						`post_id` int( 11 ) NOT NULL default '0',
						`final` varchar( 150 ) NOT NULL default '',
						`custom` longtext NOT NULL,
						PRIMARY KEY ( `id` )) $charset_collate;";
		maybe_create_table( $wpdb->leaguemanager_matches, $create_matches_sql );

		/*
		$create_stats_sql = "CREATE TABLE {$wpdb->leaguemanager_stats} (
						`id` int( 11 ) NOT NULL AUTO_INCREMENT,
						`name` varchar( 30 ) NOT NULL default '',
						`fields` longtext NOT NULL,
						`league_id` int( 11 ) NOT NULL,
						PRIMARY KEY ( `id` )) $charset_collate;";
		maybe_create_table( $wpdb->leaguemanager_stats, $create_stats_sql );
		*/
	}


	/**
	 * Uninstall Plugin
	 */
	static function uninstall() {
		global $wpdb;

		$wpdb->query( "DROP TABLE {$wpdb->leaguemanager_matches}" );
		$wpdb->query( "DROP TABLE {$wpdb->leaguemanager_teams}" );
		//$wpdb->query( "DROP TABLE {$wpdb->leaguemanager_stats}" );
		$wpdb->query( "DROP TABLE {$wpdb->leaguemanager}" );

		delete_option( 'leaguemanager_widget' );
		delete_option( 'leaguemanager' );

		/*
		* Set Capabilities
		*/
		$role = get_role('administrator');
		if ( $role !== null ) {
			$role->remove_cap('leaguemanager_settings');
			$role->remove_cap('view_leagues');
			$role->remove_cap('edit_leagues');
			$role->remove_cap('edit_league_settings');
			$role->remove_cap('del_leagues');
			$role->remove_cap('edit_seasons');
			$role->remove_cap('del_seasons');
			$role->remove_cap('edit_teams');
			$role->remove_cap('del_teams');
			$role->remove_cap('edit_matches');
			$role->remove_cap('del_matches');
			$role->remove_cap('update_results');
			$role->remove_cap('export_leagues');
			$role->remove_cap('import_leagues');
			$role->remove_cap('manage_leaguemanager');
			
			// old rules
			$role->remove_cap('leaguemanager');
			$role->remove_cap('league_manager'); // temporary rule
		}
		
		$role = get_role('editor');
		if ( $role !== null ) {
			$role->remove_cap('view_leagues');
			
			// old rules
			$role->remove_cap('leaguemanager');
		}
		
		// Delete media files
		self::removeDir(self::getFilePath());
	}

	
	/**
	 * recursively remove directory
	 *
	 * @param string $dir
	 */
	public function removeDir($dir) {
		if ( file_exists($dir) && is_dir($dir) ) {
			$files = array_diff(scandir($dir), array('.','..'));
			foreach ($files AS $file) {
				if (is_dir("$dir/$file"))
					$this->removeDir("$dir/$file");
				else
					@unlink("$dir/$file");
			}
			@rmdir($dir);
		}
	}
	
	
	/**
	 * get months in appropriate language depending on Wordpress locale
	 *
	 * @return array
	 */
	public function getMonths() {
		$locale = explode("_", get_locale());
		setlocale( LC_ALL, $locale[0] );
		$months = array();
		for ( $month = 1; $month <= 12; $month++ )
			$months[$month] = utf8_encode(strftime( "%B", mktime( 0,0,0, $month, date("j"), date("Y") ) ));
		
		return $months;
	}
	
	
	/**
	 * set a message
	 *
	 * @param string $message
	 * @param boolean $error
	 */
	public function setMessage( $message, $error = false ) {
		$this->error = $error;
		$this->message = $message;
	}
	

	/**
	 * get message
	 *
	 * @return string
	 */
	public function getMessage() {
		return $this->message;
	}
	
	
	/**
	 * print formatted success or error message
	 */
	public function printMessage() {
		if (!empty($this->message)) {
			if ( $this->error )
				echo "<div class='box error'><p>".$this->message."</p></div>";
			else
				echo "<div class='box success updated fade'><p><strong>".$this->message."</strong></p></div>";
		}
		
		$this->message = '';
	}
	
	
	/**
	 * get leagues from database
	 *
	 * @return array
	 */
	public function getLeagues() {
		global $wpdb;
		
		$leagues = $wpdb->get_results("SELECT `title`, `id`, `settings`, `seasons` FROM {$wpdb->leaguemanager} ORDER BY id ASC"); $class = '';
		foreach ( $leagues AS $i => $league ) {
			$class = ( 'alternate' == $class ) ? '' : 'alternate';
			
			$league->cssClass = $class;
			$league->title = stripslashes($league->title);
			$league->seasons = maybe_unserialize($league->seasons);
			$league->settings = maybe_unserialize($league->settings);

			$league = (object)array_merge((array)$league, $league->settings);
			//unset($league->settings, $league->settings);
			
			$leagues[$i] = $league;
		}
		
		return $leagues;
	}
	
	
	/**
	 * get league types
	 *
	 * @return array
	 */
	public function getLeagueTypes() {
		$types = array( 'default' => __('Default', 'leaguemanager') );
		/**
		 * Add custom league types
		 *
		 * @param array $types
		 * @return array
		 * @category wp-filter
		 */
		$types = apply_filters('leaguemanager_sports', $types);
		asort($types);

		return $types;
	}
	
	
	/**
	 * get card name
	 *
	 * @param string $type
	 * @return string
	 */
	public function getCards( $type = false ) {
		$cards = array( 'red' => __( 'Red', 'leaguemanager' ), 'yellow' => __( 'Yellow', 'leaguemanager' ), 'yellow-red' => __( 'Yellow/Red', 'leaguemanager' ) );
		/**
		 * Add custom card types
		 *
		 * @param array $cards
		 * @return array
		 * @category wp-filter
		 */
		$cards = apply_filters( 'leaguemanager_cards', $cards );

		if ( $type )
			return $cards[$type];
		else
			return $cards;
	}
	
	
	/**
	 * small helper function to correct 0:0 in overtime fields to empty values
	 */
	public function cleanOvertime() {
		global $wpdb;
		
		// get all matches from database
		$matches = $wpdb->get_results( "SELECT * FROM {$wpdb->leaguemanager_matches} ORDER By id ASC" );
		foreach ( $matches AS $match ) {
			$match->custom = stripslashes_deep(maybe_unserialize($match->custom));
			
			if ( count($match->custom) && $match->custom['overtime']['home'] == "0" && $match->custom['overtime']['away'] == "0" ) {
				$match->custom['overtime'] = array('home' => '', 'away' => '');
			}
			$wpdb->query($wpdb->prepare( "UPDATE {$wpdb->leaguemanager_matches} SET `custom` = '%s' WHERE `id` = '%d'", maybe_serialize($match->custom), $match->id ));
		}
	}
	
	
	/**
	 * archive files in zip format
	 *
	 * @param array $files
	 * @param string $destination
	 * @param boolean $overwrite
	 * @return boolean
	 */
	public function createZip($files, $destination, $overwrite = true) {
		//if the zip file already exists and overwrite is false, return false
		if(file_exists($destination) && !$overwrite)
			return false;
	
		if (!file_exists($destination))
			$overwrite = false;
		
		$valid_files = array();
		
		// make sure that files are an array
		if (!is_array($files)) $files = array($files);
		
		foreach ($files AS $file) {
			if (file_exists($file))
				$valid_files[] = $file;
		}
		
		// Only proceed if we have valid files
		if (count($valid_files)) {
			// create zip Archive
			$zip = new ZipArchive();
			$res = $zip->open($destination, $overwrite ? ZIPARCHIVE::OVERWRITE : ZipArchive::CREATE);

			if ($res == TRUE) {
				foreach ($valid_files AS $file) {
					$zip->addFile($file, basename($file));
				}

				$zip->close();
			} else {
				return false;
			}
			
			return file_exists($destination);
		} else {
			return false;
		}
	}

	
	/**
	 * unzip media files to upload folder
	 *
	 * @param string $zip_file
	 * @param string|null $destination destination directory. Default: global projectmanager upload path
	 * @return boolean
	 */
	public function unzipFiles($zip_file, $destination = NULL) {
		$zip = new ZipArchive();
		$res = $zip->open($zip_file);
		
		if (is_null($destination)) $destination = $this->getFilePath();
		
		if ($res == TRUE) {
			$zip->extractTo($destination);
			$zip->close();
			
			return true;
		} else {
			return false;
		}
	}
}


global $leaguemanager;
if ( is_admin() ) {
	require_once (dirname (__FILE__) . '/admin/admin.php');
	$leaguemanager = new LeagueManagerAdmin();
} else {
	$leaguemanager = new LeagueManager();
}

// suppress output for export
if ( isset($_POST['leaguemanager_export']) )
	ob_start();
?>