<?php
/**
 * Champinship object
 *
 */
 
add_filter( 'leaguemanager_modes', 'leaguemanager_championship_mode' );
/**
 * add championship mode
 *
 * @param array $modes
 * @return array
 */
function leaguemanager_championship_mode( $modes ) {
	$modes['championship'] = __( 'Championship', 'leaguemanager' );
	return $modes;
}


/**
 * Implement Championship mode
 *
 * @package	LeagueManager
 * @subpackage League_Championship
*/
final class League_Championship extends LeagueManager {
	/**
	 * League ID
	 *
	 * @var int
	 */
	public $league_id = 0;
	
	/**
	 * preliminary groups
	 *
	 * @var array
	 */
	public $groups = array();
	
	/**
	 * number of preliminary groups
	 *
	 * @var int
	 */
	public $num_groups = 0;
	
	/**
	 * number of teams per group
	 *
	 * @var int
	 */
	public $teams_per_group = 0;
	
	/**
	 * number of teams to advance to final rounds
	 *
	 * @var int
	 */
	public $num_advance = 0;
	
	/**
	 * number of final rounds
	 *
	 * @var int
	 */
	public $num_rounds = 0;

	/**
	 * number of teams in first round
	 *
	 * @var int
	 */
	public $num_teams_first_round = 0;

	/**
	 * final keys indexed by round
	 *
	 * @var array
	 */
	private $keys = array();

	/**
	 * finals indexed by key
	 *
	 * @var array
	 */
	public $finals = array();

	/**
	 * current final key
	 *
	 * @var string
	 */
	public $current_final = '';
	
	/**
	 * array of final team names
	 *
	 * @var array
	 */
	public $final_teams = array();
	
	/**
	 * image of cup icon
	 *
	 * @var string
	 */
	public $cup_icon = '';
	
	
	/**
	 * initialize Championship Mode
	 *
	 * @param int $league_id League ID
	 * @param array $settings
	 */
	public function __construct( $league_id, $settings ) {
		add_action( 'league_settings_championship', array(&$this, 'settingsPage') );
		
		$this->league_id = intval($league_id);
		if (isset($settings['groups'])) $this->groups = $settings['groups'];
		$this->teams_per_group = isset($settings['teams_per_group']) ? intval($settings['teams_per_group']) : 4;
		$this->num_groups = count($this->groups);
		$this->num_advance = isset($settings['num_advance']) ? $settings['num_advance'] : 0;
		$this->num_teams_first_round = $this->num_groups * $this->num_advance;
		$this->num_rounds = log($this->num_teams_first_round, 2);
		
		$num_teams = 2;
		$i = $this->num_rounds;
		while ( $num_teams <= $this->num_teams_first_round ) {
			$finalkey = $this->getFinalKey( $num_teams );
			
			$num_matches = $num_teams/2;
			$is_final = ( $finalkey == 'final' ) ? true : false;
			$this->finals[$finalkey] = array(
				'key' => $finalkey,
				'is_final' => $is_final,
				'name' => $this->getFinalName($finalkey),
				'num_matches' => $num_matches,
				'num_teams' => $num_teams,
				'colspan' => ( $this->num_teams_first_round/2 >= 4 ) ? ceil(4/$num_matches) : ceil(($this->num_teams_first_round/2)/$num_matches),
				'round' => $i
			);

			// Separately add match for third place
			if ( $num_teams == 2 && (isset($settings['match_place3']) && $settings['match_place3'] == 1) ) {
				$finalkey = 'third';
				$this->finals[$finalkey] = array(
					'key' => $finalkey,
					'is_final' => false,
					'name' => $this->getFinalName($finalkey),
					'num_matches' => $num_matches,
					'num_teams' => $num_teams,
					'colspan' => ( $this->num_teams_first_round/2 >= 4 ) ? ceil(4/$num_matches) : ceil(($this->num_teams_first_round/2)/$num_matches),
					'round' => $i
				);
			}

			$this->keys[$i] = $finalkey;

			$i--;
			$num_teams = $num_teams * 2;
		}
		
		$this->setCurrentFinal();
		$this->setFinalTeams();
		
		$this->cup_icon = '<img style="vertical-align: middle;" src="'.LEAGUEMANAGER_URL . '/admin/icons/cup.png" />';
	}


	/**
	 * get groups
	 *
	 * @return array
	 */
	public function getGroups() {
		return $this->groups;
	}


	/**
	 * get final key
	 *
	 * @param int $round
	 * @return string
	 */
	public function getFinalKeys( $round = false ) {
		if ($round) {
			if ( isset($this->keys[$round]) )
				return $this->keys[$round];
			
			return false;
		}
		
		return $this->keys;
	}


	/**
	 * get final data
	 *
	 * @param string $key
	 * @return mixed
	 */
	public function getFinals( $key = false ) {
		if ( $key == 'current' )
			$key = $this->current_final;
		
		if ( $key )
			return $this->finals[$key];

		return $this->finals;
	}


	/**
	 * add settings
	 *
	 * @param League $league
	 */
	public function settingsPage( $league ) {
		echo '<tr valign="top">';
		echo '<th scope="row"><label for="groups">'.__( 'Groups', 'leaguemanager' ).'</label></th>';
		echo '<td valign="top"><input type="text" name="settings[groups]" id="groups" size="20" value="'.((isset($league->groups)) ? is_array($league->groups) ? implode(";",$league->groups) : $league->groups  : '').'" />&#160;<span class="setting-description">'.__( 'Separate Groups by semicolon ;', 'leaguemanager' ).'</span></td>';
		echo '</tr>';
		echo '<th scope="row"><label for="teams_per_group">'.__( 'Teams per group', 'leaguemanager' ).'</label></th>';
		echo '<td valign="top"><input type="text" name="settings[teams_per_group]" id="teams_per_group" size="20" value="'.((isset($league->teams_per_group)) ? $league->teams_per_group  : '').'" /></td>';
		echo '</tr>';
		echo '<tr valign="top">';
		echo '<th scope="row"><label for="num_advance">'.__('Teams Advance', 'leaguemanager').'</label></th>';
		echo '<td><input type="text" size="3" id="num_advance" name="settings[num_advance]" value="'.((isset($league->num_advance)) ? $league->num_advance  : '').'" /></td>';
		echo "<tr valign='top'>";
		echo "<th scope='row'><label for='match_place3'>".__('Include 3rd place match', 'leaguemanager' )."</label></th>";
			$checked = (isset($league->match_place3) && 1 == $league->match_place3 ) ? ' checked="checked"' : '';
		echo "<td><input type='checkbox' id='match_place3' name='settings[match_place3]' value='1'".$checked." /></td>";
		echo "</tr>";
		echo "<tr valign='top'>";
		echo "<th scope='row'><label for='non_group'>".__('Allow Non-Group Games', 'leaguemanager' )."</label></th>";
			$checked = (isset($league->non_group) && 1 == $league->non_group ) ? ' checked="checked"' : '';
		echo "<td><input type='checkbox' id='non_group' name='settings[non_group]' value='1'".$checked." /></td>";
		echo "</tr>";
		echo '</tr>';
	}


	/**
	 * get name of final depending on number of teams
	 *
	 * @param string $key
	 * @return string the name
	 */
	public function getFinalName( $key = false ) {
		if (empty($key)) $key = $this->current_final;
		
		if(!empty($key)) {
			if ( 'final' == $key )
				return __( 'Final', 'leaguemanager' );
			elseif ( 'third' == $key )
				return __( 'Third Place', 'leaguemanager' );
			elseif ( 'semi' == $key )
				return __( 'Semi Final', 'leaguemanager' );
			elseif ( 'quarter' == $key )
				return __( 'Quarter Final', 'leaguemanager' );
			else {
				$tmp = explode("-", $key);
				return sprintf(__( 'Last-%d', 'leaguemanager'), $tmp[1]);
			}
		}
	}


	/**
	 * get key of final depending on number of teams
	 *
	 * @param int $num_teams
	 * @return string the key
	 */
	private function getFinalKey( $num_teams ) {
		if ( 2 == $num_teams )
			return 'final';
		elseif ( 4 == $num_teams )
			return 'semi';
		elseif ( 8 == $num_teams )
			return 'quarter';
		else
			return 'last-'.$num_teams;
	}

	
	/**
	 * set current final key
	 *
	 * @param string $final
	 */
	public function setCurrentFinal($final = false) {
		if ( isset($_GET['final']) )
			$key = htmlspecialchars($_GET['final']);
		elseif ( $final )
			$key = htmlspecialchars($final);
		else
			$key = $this->getFinalKeys(1);
		
		$this->current_final =  $key;
	}
	
	
	/**
	 * get current final key
	 *
	 * @return string
	 */
	public function getCurrentFinalKey() {
		return $this->current_final;
	}
	
	
	/**
	 * get number of matches for specific final
	 *
	 * @param string $finalkey
	 * @return int
	 */
	public function getNumMatches( $finalkey ) {
		if ( isset($this->finals[$finalkey]) )
			return $this->finals[$finalkey]['num_matches'];
		
		return 0;
	}
	
	
	/**
	 * set general names for final rounds
	 */
	private function setFinalTeams() {
		// Final Rounds
		foreach ( $this->getFinals() AS $k => $data ) {
			$this->final_teams[$k] = array();

			if ($data['round'] > 1) {
				// get data of previous round
				$final = $this->getFinals( $this->getFinalKeys($data['round']-1) );
					
				for ( $x = 1; $x <= $final['num_matches']; $x++ ) {
					if ( $k == 'third' ) {
						$title = sprintf(__('Looser %s %d', 'leaguemanager'), $final['name'], $x);
						$key = '2_'.$final['key'].'_'.$x;
					} else {
						$title = sprintf(__('Winner %s %d', 'leaguemanager'), $final['name'], $x);
						$key = '1_'.$final['key'].'_'.$x;
					}

					$this->final_teams[$k][$key] = (object) array('id' => $key, 'title' => $title, 'home' => 0);
				}
			} else {
				// First Final Rounds
				foreach ( $this->groups AS $group ) {
					for ( $a = 1; $a <= $this->num_advance; $a++ ) {
						$this->final_teams[$k][$a.'_'.$group] =	(object) array('id' => $a.'_'.$group, 'title' => sprintf(__('%d. Group %s', 'leaguemanager'), $a, $group), 'home' => 0);
					}
				}
			}
		}
	}


	/**
	 * get final team names
	 *
	 * @param string $final
	 * @return array
	 */
	public function getFinalTeams( $final ) {
		//$final = $this->getFinals($final);
		//$key = $final['round'] > 1 ? $this->getFinalKeys($final['round']-1) : 'preliminary';
		
		return $this->final_teams[$final];
	}
	
	
	/**
	 * update final rounds results
	 *
	 * @param array $matches
	 * @param array $home_points
	 * @param array $away_points
	 * @param array $home_team
	 * @param array $away_team
	 * @param array $status
	 * @param array $custom
	 * @param int $round
	 */
	public function updateFinalResults( $matches, $home_points, $away_points, $home_team, $away_team, $status, $custom, $round ) {
		if ( is_admin() && current_user_can( 'update_results' ) ) {
			$league = get_league();
			$league->setFinals(true);
			$league->_updateResults($matches, $home_points, $away_points, $home_team, $away_team, $status, $custom);
			
			if ( $round < $this->num_rounds )
				$this->proceed($this->getFinalKeys($round), $this->getFinalKeys($round+1));
			
			$this->setMessage( sprintf(__('Updated Results of %d matches','leaguemanager'), count($matches)) );
		}
	}


	/**
	 * start final rounds
	 *
	 */
	public function startFinalRounds() {
		if ( is_admin() && current_user_can( 'update_results' ) ) {
			global $wpdb;
			
			$league = get_league();
			$matches = $league->getMatches( array("final" => $this->getFinalKeys(1), "limit" => false, "match_day" => -1, "reset_query_args" => true) );
			foreach ( $matches AS $match ) {
				$update = true;
				$home = explode("_", $match->home_team);
				$away = explode("_", $match->away_team);
				
				$home = array( 'rank' => $home[0], 'group' => $home[1] );
				$away = array( 'rank' => $away[0], 'group' => $away[1] );
				
				$home_team = $league->getTeams( array("rank" => $home['rank'], "group" => $home['group']) );
				$away_team = $league->getTeams( array("rank" => $away['rank'], "group" => $away['group']) );

				if ( $home_team && $away_team ) {
					$home['team'] = $home_team[0]->id;
					$away['team'] = $away_team[0]->id;
				} else {
					$update = false;
				}

				if ( $update ) {
					$wpdb->query( $wpdb->prepare( "UPDATE {$wpdb->leaguemanager_matches} SET `home_team` = %d, `away_team` = %d WHERE `id` = %d", $home['team'], $away['team'], $match->id ) );
				}
			}
		}
	}
	
	
	/**
	 * proceed to next final round
	 *
	 * @param string $last
	 * @param string $current
	 * @return void
	 */
	private function proceed( $last, $current ) {
		global $wpdb;

		$league = get_league();
		$matches = $league->getMatches( array("final" => $current, "limit" => false) );
		foreach ( $matches AS $match ) {
			$update = true;
			$home = explode("_", $match->home_team);
			$away = explode("_", $match->away_team);
			
			if ( is_array($home) && is_array($away) ) {
				if ( isset($home[1]) && isset($away[1]) ) {
					$col = ( $home[0] == 1 ) ? 'winner_id' : 'loser_id';
					$home = array( 'col' => $col, 'finalkey' => $home[1], 'no' => $home[2] );
					$col = ( $away[0] == 1 ) ? 'winner_id' : 'loser_id';
					$away = array( 'col' => $col, 'finalkey' => $away[1], 'no' => $away[2] );
				} else {
					$home['no'] = $away['no'] = 0;
				}
				
				// get matches of previous round
				$prev = $league->getMatches( array("final" => $last, "limit" => false) );
				if ( isset($prev[$home['no']-1]) && isset($prev[$away['no']-1]) ) {
					$prev_home = $prev[$home['no']-1];
					$prev_away = $prev[$away['no']-1];

					$home['team'] = $prev_home->{$home['col']};
					$away['team'] = $prev_away->{$away['col']};
					
					// don't update if winners are not defined (e.g. results not there yet)
					if ( intval($home['team']) <= 0 || intval($away['team']) <= 0 )
						$update = false;
				} else {
					$update = false;
				}

				if ( $update ) {
					$wpdb->query( $wpdb->prepare( "UPDATE {$wpdb->leaguemanager_matches} SET `home_team` = %d, `away_team` = %d WHERE `id` = %d", $home['team'], $away['team'], $match->id ) );
					// Set winners on final
					if ( $current == 'third' ) {
						$match = $league->getMatches( array("final" => "final", "limit" => false) );
						$match = $match[0];
						$home_team = $prev_home->winner_id;
						$away_team = $prev_away->winner_id;
						if ( intval($home_team) > 0 && intval($away_team) > 0 ) {
							$wpdb->query( $wpdb->prepare( "UPDATE {$wpdb->leaguemanager_matches} SET `home_team`= %d, `away_team`= %d WHERE `id` = %d", $home_team, $away_team, $match->id ) );
						}
					}
				}
			}
		}
	}
	
	
	/**
	 * display administration panel
	 *
	 */
	 public function displayAdminPage() {
		 if ( is_admin() && current_user_can('view_leagues') ) {
			$league = get_league();

			if ( isset($_POST['startFinals']) ) {
				if ( current_user_can( 'update_results' ) )
					$this->startFinalRounds($league->id);
				else
					echo '<div class="error"><p style="text-align: center;">'.__("You do not have sufficient permissions to access this page.").'</p></div>';
			}

			if ( isset($_POST['updateFinalResults']) ) {
				if ( current_user_can( 'update_results' ) ) {
					if ( !is_numeric(end($_POST['home_team'])) ) {
						$this->setMessage(__( "It seems the previous round is not over yet.", 'leaguemanager'), true);
						$this->printMessage();
					} else {
						$custom = isset($_POST['custom']) ? $_POST['custom'] : false;
						$this->updateFinalResults($_POST['matches'], $_POST['home_points'], $_POST['away_points'], $_POST['home_team'], $_POST['away_team'], $_POST['status'], $custom, $_POST['round']);
					}
				} else {
					$this->setMessage(__("You do not have sufficient permissions to access this page."), true);
				}
				$this->printMessage();
			}
			
			$class = 'alternate';
			if (count($this->groups) > 0) $league->setGroup($this->groups[0]);
			
			$tab = 0;
			if (isset($_GET['jquery-ui-tab'])) $tab = intval($_GET['jquery-ui-tab']);
			if (isset($_POST['jquery-ui-tab'])) $tab = intval($_POST['jquery-ui-tab']);
			
			include_once( LEAGUEMANAGER_PATH . '/admin/championship.php' );
		 } else {
			 echo '<div class="error"><p style="text-align: center;">'.__("You do not have sufficient permissions to access this page.").'</p></div>';
		 }
	 }
}
?>