<?php
/**
 * League_Gymnastics API: League_Gymnastics class
 *
 * @package LeagueManager
 * @subpackage League_Gymnastics
 */
 
 
add_filter( 'leaguemanager_sports', 'leaguemanager_sports_gymnastics' );
/**
 * add gymnastics to list
 *
 * @param array $sports
 * @return array
 */
function leaguemanager_sports_gymnastics( $sports ) {
	$sports['gymnastics'] = __( 'Gymnastics', 'leaguemanager' );
	return $sports;
}

 
/**
 * Class to implement the League_Gymnastics object
 *
 */
final class League_Gymnastics extends League {
	/**
	 * sports key
	 *
	 * @var string
	 */
	public $sport = 'gymnastics';

	/**
	 * apparatus keys and translated labels
	 *
	 * @var array
	 */
	private $apparatus = array();
	
	/**
	 * default apparatus points are zero
	 *
	 * @var array
	 */
	private $default_apparatus_points = array();
	
	
	/**
	 * load custom setings
	 *
	 * @param object $league League object
	 */
	public function __construct($league) {
		parent::__construct($league);
		
		// save apparatus keys and translated labels
		$apparatus = array( 'floor' => __( 'Floor', 'leaguemanager' ), 'pommelhorse' => __( 'Pommelhorse', 'leaguemanager' ), 'rings' => __( 'Rings', 'leaguemanager' ), 'vault' => __( 'Vault', 'leaguemanager' ), 'parallelbars' => __( 'Parallel bars', 'leaguemanager' ), 'highbars' => __( 'High bars', 'leaguemanager' ) );
		
		foreach ( $apparatus AS $key => $label ) {
			$this->apparatus[$key] = $label;
			
			// Initialize default apparatus points
			$this->default_apparatus_points[$key] = 0;
			
			$this->fields_team[$key] = array('label' => $label, 'type' => 'input');
			$this->fields_match[$key] = array('label' => $label, 'keys' => array('home', 'guest'));
		}

		$this->fields_match['apparatus_points'] = array('label' => __( 'Apparatus Points', 'leaguemanager' ), 'keys' => array('plus', 'minus'));
		$this->fields_team['points2'] = array('label' => __( 'AP', 'leaguemanager' ), 'desc' => __('Apparatus Points', 'leaguemanager'), 'keys' => array('plus', 'minus'), 'type' => 'input');
		$this->fields_team['diff'] = array('label' => __( 'Diff', 'leaguemanager' ), 'desc' => __('Apparatus Points Difference', 'leaguemanager'));
	}
	
	
	/**
	 * rank Teams
	 * 1. Primary points DESC
	 * 2. Secondary Point Difference DESC
	 * 3. Win Percentage DESC
	 * 4. Done Matches ASC
	 *
	 * @param array $teams
	 * @return array
	 */
	protected function rankTeams( $teams ) {
		$points = $points2 = $done = $diff = $WinPerc = array();
		foreach ( $teams AS $key => $row ) {
			$points[$key] = $row->points['plus'];
			$points2[$key] = $row->points2['plus'];
			$done[$key] = $row->done_matches;
			$diff[$key] = $row->diff;
			$WinPerc[$key] = $row->winPercent;
		}

		array_multisort( $points, SORT_DESC, $diff, SORT_DESC, $WinPerc, SORT_DESC, $done, SORT_ASC, $teams );
		return $teams;
	}


	/**
	 * update match results and automatically calculate apparatus points and score
	 *
	 * @param Match $match
	 * @return Match
	 */
	protected function updateResults( $match ) {
		$force = true;
		if (is_admin() && current_user_can('update_results')) {
			// update primary points
			if ( $force || $match->home_points == "" && $match->away_points == "" ) {
				$home_points = $away_points = 0;
				foreach ( $this->apparatus AS $key => $label ) {
					$home_points += $match->{$key}['home'];
					$away_points += $match->{$key}['guest'];
				}

				if ( $home_points == 0 && $away_points == 0 )
					$home_points = $away_points = '';
				
				$match->home_points = $home_points;
				$match->away_points = $away_points;
			}
			
			// update apparatus points
			if ( $force || $custom['apparatus_points']['plus'] == '' && $custom['apparatus_points']['minus'] == '' ) {
				$ap = array('plus' => 0, 'minus' => 0);
				foreach ( $this->apparatus AS $key => $label ) {
					if ( $match->{$key}['home'] != '' && $match->{$key}['guest'] != '' ) {
						if ( $match->{$key}['home'] > $match->{$key}['guest'] ) {
							$ap['plus'] += 2;
						} elseif ( $match->{$key}['home'] < $match->{$key}['guest'] ) {
							$ap['minus'] += 2;
						} else {
							$ap['plus'] += 1;
							$ap['minus'] += 1;
						}
					}
				}
				if ( $ap['plus'] == 0 && $ap['minus'] == 0 ) $ap = array( 'plus' => '', 'minus' => '' );
					
				$match->custom['apparatus_points'] = $ap;
			}
		}
		
		return $match;
	}
	
	
	/**
	 * get standings data for given team
	 *
	 * @param Team $team
	 * @param array $data
	 * @param array|false $matches
	 * @return array
	 */
	protected function getStandingsData( $team, $data, $matches = false ) {
		foreach ( $this->apparatus AS $key => $label )
			$data[$key] = 0;
	
		if ( !$matches ) $matches = $this->getMatches( array("team_id" => $team->id, "limit" => false, "cache" => false, "match_day" => -1, "reset_query_args" => true) );
		foreach ( $matches AS $match ) {
			if ( isset($match->floor) ) {
				foreach ($this->apparatus AS $key => $value) {
					$data[$key] += $this->getApparatusPoints($match, $team->id, $key);
				}
			}
		}

		return $data;
	}
	
	
	/**
	 * calculate apparatus points
	 *
	 * @param Team $team
	 * @param array $matches
	 * @return array
	 */
	protected function calculateSecondaryPoints( $team, $matches = false ) {
		$points = array( 'plus' => 0, 'minus' => 0);
		
		if (!$matches) $matches =  $this->getMatches( array("team_id" => $team->id, "match_day" => -1, "limit" => false, "cache" => false, "reset_query_args" => true) );
		if ( $matches ) {
			foreach ( $matches AS $match ) {
				$custom = (array)maybe_unserialize($match->custom);
					
				if (!isset($custom['apparatus_points'])) $custom['apparatus_points'] = array('plus' => '', 'minus' => '');
				if ( $match->home_team == $team->id ) {
					$points['plus'] += intval($custom['apparatus_points']['plus']);
					$points['minus'] += intval($custom['apparatus_points']['minus']);
				}
				if ( $match->away_team == $team->id ) {
					$points['plus'] += intval($custom['apparatus_points']['minus']);
					$points['minus'] += intval($custom['apparatus_points']['plus']);
				}
			}
		}
		
		return $points;
	}

	
	/**
	 * get results for single apparatus
	 *
	 * @param Match $match
	 * @param int $team_id
	 * @param string $key
	 * @return int the number of points for apparatus
	 */
	private function getApparatusPoints($match, $team_id, $key) {
		$points = array( 'win' => 2, 'tie' => 1, 'lost' => 0);
	
		// Home Match
		if ( $team_id == $match->home_team ) {
			if ($match->{$key}['home'] > $match->{$key}['guest'])
				return $points['win'];
			elseif ($match->{$key}['home'] == $match->{$key}['guest'])
				return $points['tie'];
			else
				return $points['lost'];
		} else {
			if ($match->{$key}['home'] > $match->{$key}['guest'])
				return $points['lost'];
			elseif ($match->{$key}['home'] == $match->{$key}['guest'])
				return $points['tie'];
			else
				return $points['win'];
		}
	}
}
?>