<?php
/**
 * League_Hockey API: League_Hockey class
 *
 * @package LeagueManager
 * @subpackage League_Hockey
 */
 
 
add_filter( 'leaguemanager_sports', 'leaguemanager_sports_hockey' );
/**
 * add hockey to list
 *
 * @param array $sports
 * @return array
 */
function leaguemanager_sports_hockey( $sports ) {
	$sports['hockey'] = __( 'Hockey', 'leaguemanager' );
	return $sports;
}

/**
 * Class to implement the League_Hockey object
 *
 */
final class League_Hockey extends League {
	/**
	 * sports keys
	 *
	 * @var string
	 */
	public $sport = 'hockey';

	/**
	 * hockey types
	 *
	 * @var array
	 */
	private $keys = array();
	
	
	/**
	 * load specific settings
	 *
	 * @param object $league League object
	 */
	public function __construct($league) {
		parent::__construct($league);
		
		$this->keys = array( 'del' => __( 'German Icehockey League (DEL)', 'leaguemanager' ), 'nhl' => __( 'National Hockey League (NHL)', 'leaguemanager' ) );
		
		$this->fields_match['thirds'] = array( 'label' => __( 'Thirds', 'leaguemanager' ), 'keys' => array() );
		for ( $i = 1; $i <= 3; $i++ ) {
			$this->fields_match['thirds']['keys'][$i] = array('plus','minus');
		}
		$fields = array( 'overtime' => __( 'Overtime', 'leaguemanager' ), 'penalty' => __( 'Penalty', 'leaguemanager' ) );
		foreach ( $fields AS $key => $label ) {			
			if ( $key == "halftime" )
				$keys = array('plus', 'minus');
			else
				$keys = array('home', 'away');
			
			$this->fields_match[$key] = array('label' => $label, 'keys' => $keys);
		}
		
		$this->fields_team['points2'] = array('label' => __( 'Goals', 'leaguemanager' ), 'keys' => array('plus', 'minus'), 'type' => 'input');
		$this->fields_team['diff'] = array('label' => __( 'Diff', 'leaguemanager' ), 'desc' => __('Goals Difference', 'leaguemanager'));
		
		add_filter( 'leaguemanager_point_rules_list', array(&$this, 'getPointRuleList') );
		add_filter( 'leaguemanager_point_rules',  array(&$this, 'getPointRules') );
		add_action( 'leaguemanager_doc_point_rules', array(&$this, 'pointRuleDocumentation') );
		
		add_filter( 'team_points_'.$key, array(&$this, 'calculatePoints'), 10, 4 );
	}


	/**
	 * get Point Rule list
	 *
	 * @param array $rules
	 * @return array
	 */
	public function getPointRuleList( $rules ) {
		foreach ( $this->keys AS $key => $name )
			$rules[$key] = $name;

		return $rules;
	}


	/**
	 * get Point rules
	 *
	 * @param array $rules
	 * @return array
	 */
	public function getPointRules( $rules ) {
		// DEL rule
		$rules['del'] = array( 'forwin' => 3, 'fordraw' => 1, 'forloss' => 0, 'forwin_overtime' => 2, 'forloss_overtime' => 1 );
		// NHL rule
		$rules['nhl'] = array( 'forwin' => 2, 'fordraw' => 0, 'forloss' => 0, 'forwin_overtime' => 2, 'forloss_overtime' => 1 );

		return $rules;
	}


	/**
	 * rank Teams
	 * 1. Primary points
	 * 2. Goals difference
	 * 3. Done Matches
	 * 4. Goals
	 *
	 * @param array $teams
	 * @return array of teams
	 */
	protected function rankTeams( $teams ) {
		$points = $done = $diff = $goals = array();
		foreach ( $teams AS $key => $row ) {
			$points[$key] = $row->points['plus'];
			$done[$key] = $row->done_matches;
			$diff[$key] = $row->diff;
			$goals[$key] = $row->points2['plus'];
		}

		array_multisort( $points, SORT_DESC, $diff, SORT_DESC, $done, SORT_ASC, $goals, SORT_DESC, $teams );
		return $teams;
	}


	/**
	 * display point  rule documentation
	 *
	 */
	public function pointRuleDocumentation() {
		echo '<h4>'.__( 'German Icehockey League (DEL)', 'leaguemanager' ).'</h4>';
		echo '<p>'.__( 'The DEL uses a more complicated form of the Three-Point-Rule. The winner after regular time gets three points, the loser none. The winner after overtime gets two points and the loser one. This rule was also applied at the Ice Hockey World Championship in 2008.', 'leaguemanager' ).'</p>';

		echo '<h4>'.__( 'National Hockey League (NHL)', 'leaguemanager' ).'</h4>';
		echo '<p>'.__( 'The NHL uses a derivative of the Two-Point-Rule. The winner after regular time and overtime gains two points whereas the loser after overtime and penalty gets one.', 'leaguemanager' ).'</p>';
	}


	/**
	 * re-calculate points
	 *
	 * @param array $points
	 * @param int $team_id
	 * @param array $rule
	 * @param array $matches
	 * @return array with modified points
	 */
	public function calculatePoints( $points, $team_id, $rule, $matches ) {
		extract($rule);

		$num_won_overtime = $this->getNumWonMatchesOvertime( $team_id, $matches );
		$num_lost_overtime = $this->getNumLostMatchesOvertime( $team_id, $matches );

		$points['plus'] = $points['plus'] - $num_won_overtime * $forwin + $num_won_overtime * $forwin_overtime + $num_lost_overtime * $forloss_overtime;
		$points['minus'] = $points['minus'] - $num_lost_overtime * $forwin + $num_won_overtime * $forloss_overtime + $num_lost_overtime * $forwin_overtime;

		return $points;
	}


	/**
	 * get number of won matches after overtime
	 *
	 * @param int $team_id
	 * @param array $matches
	 * @return int
	 */
	private function getNumWonMatchesOvertime( $team_id, $matches ) {
		$num = 0;
		foreach ( $matches AS $match ) {
			$custom = maybe_unserialize($match->custom);
			if ( $team_id == $match->winner_id && isset($custom['overtime']) && $custom['overtime']['home'] != '' && $custom['overtime']['away'] != '' )
				$num++;
		}
		return $num;
	}


	/**
	 * get number of lost matches after overtime
	 *
	 * @param int $team_id
	 * @param array $matches
	 * @return int
	 */
	private function getNumLostMatchesOvertime( $team_id, $matches ) {
		$num = 0;
		foreach ( $matches AS $match ) {
			$custom = maybe_unserialize($match->custom);
			if ( $team_id == $match->loser_id && isset($custom['overtime']) && $custom['overtime']['home'] != '' && $custom['overtime']['away'] != '' )
				$num++;
		}
		return $num;
	}
	
	
	/**
	 * update match results and automatically calculate apparatus points and score
	 *
	 * @param Match $match
	 * @return Match
	 */
	protected function updateResults( $match ) {
		$force = true;

		if (is_admin() && current_user_can('update_results')) {
			if ( $force || $match->home_points == "" && $match->away_points == "" ) {
				$score = array( 'home' => 0, 'guest' => 0 );
				foreach ( $match->thirds AS $third ) {
					if ( $third['plus'] != '' && $third['minus'] != '' ) {
						$score['home'] += intval($third['plus']);
						$score['guest'] += intval($third['minus']);
					}
				}
				
				if ($match->overtime['home'] != '' && $match->overtime['away'] != '') {
					$score['home'] = $score['home'] + intval($match->overtime['home']) + intval($match->penalty['home']);
					$score['guest'] = $score['guest'] + intval($match->overtime['away']) + intval($match->penalty['away']);			
				}
				
				if ( $score['home'] == 0 && $score['guest'] == 0 )
						$score['home'] = $score['guest'] = '';
					
				$match->home_points = $score['home'];
				$match->away_points = $score['guest'];
			}
		}
		
		return $match;
	}
}
?>