<?php
/**
 * League_Rugby API: League_Rugby class
 *
 * @package LeagueManager
 * @subpackage League_Rugby
 */
 
 
add_filter( 'leaguemanager_sports', 'leaguemanager_sports_rugby' );
/**
 * add rugby to list
 *
 * @param array $sports
 * @return array
 */
function leaguemanager_sports_rugby( $sports ) {
	$sports['rugby'] = __( 'Rugby', 'leaguemanager' );
	return $sports;
}

/**
 * Class to implement the League_Rugby object
 *
 */
final class League_Rugby extends League {
	/**
	 * sports keys
	 *
	 * @var string
	 */
	public $sport = 'rugby';


	/**
	 * load specific settings
	 *
	 * @param object $league League object
	 */
	public function __construct($league) {
		parent::__construct($league);
		
		$this->fields_team['gamepoints'] = array('label' => __( 'Game Points', 'leaguemanager' ), 'keys' => array('plus', 'minus'));
		
		$keys = array('tries' => __( 'Tries', 'leaguemanager' ), 'conversions' => __('Conversions','leaguemanager'), 'penaltykicks' => __('Penaltykicks', 'leaguemanager'));
		foreach ( $keys AS $key => $label ) {
			$this->fields_match[$key] = array( 'label' => $label, 'keys' => array('home', 'away') );
		}
		
		add_filter( 'team_points_'.$this->sport, array(&$this, 'calculatePoints'), 10, 4 );

		add_filter( 'leaguemanager_point_rules_list', array(&$this, 'getPointRuleList') );
		add_filter( 'leaguemanager_point_rules',  array(&$this, 'getPointRules') );
	}


	/**
	 * get Point Rule list
	 *
	 * @param array $rules
	 * @return array
	 */
	public function getPointRuleList( $rules ) {
		$rules[$this->sport] = __('Rugby', 'leaguemanager');

		return $rules;
	}


	/**
	 * get Point rules
	 *
	 * @param array $rules
	 * @return array
	 */
	public function getPointRules( $rules ) {
		$rules[$this->sport] = array( 'forwin' => 4, 'fordraw' => 2, 'forloss' => 0 );

		return $rules;
	}


	/**
	 * rank Teams
	 * 1. Points
	 * 2. Secondary points difference
	 *
	 * @param array $teams
	 * @return array of teams
	 */
	protected function rankTeams( $teams ) {
		$points = $diff = array();
		foreach ( $teams AS $key => $row ) {
			$points[$key] = $row->points['plus'];
			$diff[$key] = $row->diff;
		}

		array_multisort( $points, SORT_DESC, $diff, SORT_DESC, $teams );
		return $teams;
	}


	/**
	 * re-calculate points
	 *
	 * @param array $points
	 * @param int $team_id
	 * @param array $rule
	 * @param array $matches
	 * @return array
	 */
	public function calculatePoints( $points, $team_id, $rule, $matches ) {
		extract($rule);

		foreach ( $matches AS $match ) {
			$index = ( $match->home_team == $team_id ) ? 'home' : 'away';

			if ( $match->tries[$index] > 4 )
				$points['plus'] += 1;
			
			// Current Team Lost Match with less than 6 points
			if ( $match->loser_id == $team_id && abs($match->home_points-$match->away_points) < 6 )
				$points['plus'] += 1;
		}

		return $points;
	}


	/**
	 * update match results and automatically calculate apparatus points and score
	 *
	 * @param Match $match
	 * @return Match
	 */
	protected function updateResults( $match ) {
		$force = true;
		if (is_admin() && current_user_can('update_results')) {
			// update primary points
			if ( $force || $match->home_points == "" && $match->away_points == "" ) {
				$score['home'] = $match->tries['home'] * 5 + $match->conversions['home'] * 2 + $match->penaltykicks['home'] * 3;
				$score['away'] = $match->tries['away'] * 5 + $match->conversions['away'] * 2 + $match->penaltykicks['away'] * 3;

				if ( empty($score['home']) && empty($match->tries['away']) && empty($match->conversions['away']) && empty($match->penaltykicks['away']) ) $score['home'] = '';
				if ( empty($score['away']) && empty($match->tries['home']) && empty($match->conversions['home']) && empty($match->penaltykicks['home']) ) $score['away'] = '';

				$match->home_points = $score['home'];
				$match->away_points = $score['away'];
				
				$match->getResult( $score['home'], $score['away'], 'winner' );
				$match->getResult( $score['home'], $score['away'], 'loser' );
			}
		}

		return $match;
	}

	
	/**
	 * get standings data for given team
	 *
	 * @param Team $team
	 * @param array $data
	 * @param array|false $matches
	 * @return array
	 */
	protected function getStandingsData( $team, $data, $matches = false ) {
		$data['gamepoints'] = array( 'plus' => 0, 'minus' => 0 );

		if ( !$matches ) $matches = $this->getMatches( array("team_id" => $team->id, "limit" => false, "cache" => false, "match_day" => -1, "reset_query_args" => true) );
		foreach ( $matches AS $match ) {
			// Home Match
			if ( $team->id == $match->home_team ) {
				$data['gamepoints']['plus'] += $match->home_points;
				$data['gamepoints']['minus'] += $match->away_points;
			} else {
				$data['gamepoints']['plus'] += $match->away_points;
				$data['gamepoints']['minus'] += $match->home_points;
			}
		}
		
		return $data;
	}
}
?>