<?php
/**
 * League_Tennis API: League_Tennis class
 *
 * @package LeagueManager
 * @subpackage League_Tennis
 */
 
 
add_filter( 'leaguemanager_sports', 'leaguemanager_sports_tennis' );
/**
 * add tennis to list
 *
 * @param array $sports
 * @return array
 */
function leaguemanager_sports_tennis( $sports ) {
	$sports['tennis'] = __( 'Tennis', 'leaguemanager' );
	return $sports;
}

 
/**
 * Class to implement the League_Tennis object
 *
 */
final class League_Tennis extends League {
	/**
	 * sports key
	 *
	 * @var string
	 */
	public $sport = 'tennis';

	/**
	 * default number of sets
	 *
	 * @var int
	 */
	public $num_sets = 5;
	
	
	/**
	 * load custom setings
	 *
	 * @param object $league League object
	 */
	public function __construct($league) {
		parent::__construct($league);
		
		$this->fields_team['straight_set'] = array('label' => __( 'Straight Set', 'leaguemanager' ), 'keys' => array('win','lost'));
		$this->fields_team['split_set'] = array('label' => __( 'Split Set', 'leaguemanager' ), 'keys' => array('win','lost'));
		$this->fields_team['games_allowed'] = array('label' => __( 'Games Allowed', 'leaguemanager' ));
		
		$this->fields_match['sets'] = array( 'label' => __('Sets', 'leaguemanager'), 'keys' => array() );
		for ( $i = 1; $i <= $this->num_sets; $i++ ) {
			$this->fields_match['sets']['keys'][$i] = array('player1','player2');
		}

		add_filter( 'leaguemanager_point_rules_list', array(&$this, 'getPointRuleList') );
		add_filter( 'leaguemanager_point_rules',  array(&$this, 'getPointRules') );

		add_filter( 'team_points_'.$this->sport, array(&$this, 'calculatePoints'), 10, 4 );
		
		add_action( 'league_settings_'.$this->sport, array(&$this, 'leagueSettings') );

		add_action( 'edit_matches_header_'.$this->sport, array(&$this, 'editMatchesHeader') );
		add_action( 'edit_matches_columns_'.$this->sport, array(&$this, 'editMatchesColumns'), 10, 3 );	
		
		// filter number of matches for double games
		add_filter( 'team_done_matches', array(&$this, 'getNumDoneMatches'), 10, 2 );
		add_filter( 'team_won_matches', array(&$this, 'getNumWonMatches'), 10, 2 );
		add_filter( 'team_draw_matches', array(&$this, 'getNumTieMatches'), 10, 2 );
		add_filter( 'team_lost_matches', array(&$this, 'getNumLostMatches'), 10, 2 );
	}


	/**
	 * get Point Rule list
	 *
	 * @param array $rules
	 * @return array
	 */
	public function getPointRuleList( $rules ) {
		$rules[$this->sport] = __('Tennis', 'leaguemanager');

		return $rules;
	}


	/**
	 * get Point rules
	 *
	 * @param array $rules
	 * @return array
	 */
	public function getPointRules( $rules ) {
		$rules[$this->sport] = array( 'forwin' => 3, 'fordraw' => 0, 'forloss' => 0, 'forwin_split' => 2, 'forloss_split' => 1 );

		return $rules;
	}


	/**
	 * add league settings
	 *
	 * @param object $league
	 */
	public function leagueSettings( $league ) {
		echo "<tr valign='top'>";
		echo "<th scope='row'><label for='num_sets'>".__('Number of Sets', 'leaguemanager')."</label></th>";
		echo "<td><input type='text' name='settings[num_sets]' id='num_sets' value='".$league->num_sets."' size='3' /></td>";
		echo "</tr>";
	}


	/**
	 * Re-calculate primary points
	 *
	 * @param array $points
	 * @param int $team_id
	 * @param array $rule
	 * @param array $matches
	 * @return array
	 */
	public function calculatePoints( $points, $team_id, $rule, $matches ) {
		extract($rule);

		$team = get_team($team_id);
		$data = $this->getStandingsData($team, $matches);
		$points['plus'] = $data['straight_set']['win'] * $forwin + $data['split_set']['win'] * $forwin_split + $data['split_set']['lost'] * $forloss_split;
		$points['minus'] = $data['straight_set']['lost'] * $forwin + $data['split_set']['win'] * $forloss_split + $data['split_set']['lost'] * $forwin_split;

		return $points;
	}


	/**
	 * rank Teams
	 * 1. Points
	 * 2. Games Allowed
	 *
	 * @param array $teams
	 * @return array of teams
	 */
	protected function rankTeams( $teams ) {
		$points = $games_allowed = array();
		foreach ( $teams AS $key => $row ) {
			$points[$key] = $row->points['plus'];
			$games_allowed[$key] = $row->games_allowed;
		}

		array_multisort( $points, SORT_DESC, $games_allowed, SORT_ASC, $teams );
		return $teams;
	}
	
	
	/**
	 * get standings data for given team
	 *
	 * @param Team $team
	 * @param array $data
	 * @param array $matches
	 * @return array number of runs for and against as assoziative array
	 */
	protected function getStandingsData( $team, $data = array(), $matches = false ) {
		$data['straight_set'] = $data['split_set'] = array( 'win' => 0, 'lost' => 0 );
		$data['games_allowed'] = 0;

		if ( !$matches ) $matches = $this->getMatches( array("team_id" => $team->id, "limit" => false, "cache" => false, "match_day" => -1, "reset_query_args" => true) );
		foreach ( $matches AS $match ) {
			if ( $match->home_team == $team->id || $match->away_team == $team->id || ( isset($match->home_partner) && $match->home_partner == $team->id ) || ( isset($match->guest_partner) && $match->guest_partner == $team->id ) ) {
				if (!isset($match->home_partner)) $match->home_partner = '';
				$index = ( $team->id == $match->home_team || $team->id == $match->home_partner ) ? 'player2' : 'player1';

				// First check for Split Set, else it's straight set
				if ( $match->sets[$this->num_sets]['player1'] != '' && $match->sets[$this->num_sets]['player2'] != '' ) {
					if ( $match->winner_id == $team->id || ($team->id == $match->home_partner && $match->winner_id == $match->home_team) || ($team->id == $match->guest_partner && $match->winner_id == $match->away_team) ) {
						$data['split_set']['win'] += 1;
						for ( $j = 1; $j <= $this->num_sets-1; $j++  ) {
							$data['games_allowed'] += $match->sets[$j][$index];
						}
					} else {
						$data['split_set']['lost'] += 1;
						for ( $j = 1; $j <= $this->num_sets-1; $j++  ) {
							$data['games_allowed'] += $match->sets[$j][$index];
						}
						$data['games_allowed'] += 1;
					}
				} else {
					if (!isset($match->home_partner)) $match->home_partner = "";
					if (!isset($match->guest_partner)) $match->guest_partner = "";
					if ( $match->winner_id == $team->id || ($team->id == $match->home_partner && $match->winner_id == $match->home_team) || ($team->id == $match->guest_partner && $match->winner_id == $match->away_team) ) {
						$data['straight_set']['win'] += 1;
						for ( $j = 1; $j <= $this->num_sets-1; $j++  ) {
							$data['games_allowed'] += $match->sets[$j][$index];
						}
					} else {
						$data['straight_set']['lost'] += 1;
						for ( $j = 1; $j <= $this->num_sets-1; $j++  ) {
							$data['games_allowed'] += $match->sets[$j][$index];
						}
					}
				}
			}
		}

		return $data;
	}


	/**
	 * Add custom header to match form table
	 */
	public function editMatchesHeader() {
		echo '<th scope="col">'.__( 'Home Partner', 'leaguemanager' ).'</th><th scope="col">'.__( 'Guest Partner', 'leaguemanager' ).'</th>';
	}
	/**
	 * Add custom columns to match form table
	 *
	 * @param Match $match
	 * @param array $teams
	 * @param int $i
	 */
	public function editMatchesColumns( $match, $teams, $i ) {
		$cols = array( 'home_partner', 'guest_partner' );
		foreach ( $cols AS $col ) {
			echo '<td>';
			echo '<select name="custom['.$i.']['.$col.']" id="custom_'.$i.'_'.$col.'">';
			echo '<option value="0">'.__('None','leaguemanager').'</option>';
			foreach ( $teams AS $team ) {
				
				echo '<option value="'.$team->id.'"'.selected($team->id, $match->{$col}).'>'.$team->title.'</option>';
			}
			echo '</select>';
			echo '</td>';
		}
	}

	
	/**
	 * update match results and automatically calculate apparatus points and score
	 *
	 * @param Match $match
	 * @return Match
	 */
	protected function updateResults( $match ) {
		$force = true;
		if (is_admin() && current_user_can('update_results')) {
			// update primary points
			if ( $force || $match->home_points == "" && $match->away_points == "" ) {
				$score = array( 'home' => 0, 'guest' => 0 );
				foreach ( $match->sets AS $set ) {
					if ( $set['player1'] != '' && $set['player2'] != '' ) {
						if ( $set['player1'] > $set['player2'] ) {
							$score['home'] += 1;
						} else {
							$score['guest'] += 1;
						}
					}
				}
				
				$match->home_points = $score['home'];
				$match->away_points = $score['guest'];
			}
		}
		
		return $match;
	}
	
	
	/**
	 * get number of done matches for partners
	 *
	 * @param int $num_done
	 * @param int $team_id
	 * @return int
	 */
	public function getNumDoneMatches( $num_done, $team_id ) {
		$matches = $this->getMatches(array('team_id' => $team_id, 'limit' => false, 'home_points' => 'not_empty', 'away_points' => 'not_empty', 'match_day' => -1, 'cache' => false, 'reset_query_args' => true));
		foreach ( $matches AS $match ) {
			if ( isset($match->home_partner) && isset($match->guest_partner) ) {
				if ( $match->home_partner == $team_id || $match->guest_partner == $team_id )
					$num_done++;
			}
		}
		return $num_done;
	}


	/**
	 * get number of won matches for partners
	 *
	 * @param int $num_won
	 * @param int $team_id
	 * @return int
	 */
	public function getNumWonMatches( $num_won, $team_id ) {
		$matches = $this->getMatches(array('team_id' => $team_id, 'limit' => false, 'home_points' => 'not_empty', 'away_points' => 'not_empty', 'match_day' => -1, 'cache' => false, 'reset_query_args' => true));
		foreach ( $matches AS $match ) {
			if ( isset($match->home_partner) && isset($match->guest_partner) ) {
				if ( $match->home_partner == $team_id && $match->winner_id == $match->home_team )
					$num_won++;
				elseif ( $match->guest_partner == $team_id && $match->winner_id == $match->away_team )
					$num_won++;
					
			}
		}
		return $num_won;
	}


	/**
	 * get number of tie matches for partners
	 *
	 * @param int $num_tie
	 * @param int $team_id
	 * @return int
	 */
	public function getNumTieMatches( $num_tie, $team_id ) {
		$matches = $this->getMatches(array('team_id' => $team_id, 'limit' => false, 'winner_id' => -1, 'loser_id' => -1, 'match_day' => -1, 'cache' => false, 'reset_query_args' => true));
		foreach ( $matches AS $match ) {
			if ( isset($match->home_partner) && isset($match->guest_partner) ) {
				if ( $match->home_partner == $team_id || $match->guest_partner == $team_id )
					$num_tie++;
					
			}
		}
		return $num_tie;
	}
	

	/**
	 * get number of lost matches for partners
	 *
	 * @param int $num_lost
	 * @param int $team_id
	 * @return int
	 */
	public function getNumLostMatches( $num_lost, $team_id ) {
		$matches = $this->getMatches(array('team_id' => $team_id, 'limit' => false, 'home_points' => 'not_empty', 'away_points' => 'not_empty', 'match_day' => -1, 'cache' => false, 'reset_query_args' => true));
		foreach ( $matches AS $match ) {
			if ( isset($match->home_partner) && isset($match->guest_partner) ) {
				if ( $match->home_partner == $team_id && $match->winner_id == $match->away_team )
					$num_lost++;
				elseif ( $match->guest_partner == $team_id && $match->winner_id == $match->home_team )
					$num_lost++;
					
			}
		}
		return $num_lost;
	}
}
?>