<?php
/**
 * League_Volleyball API: League_Volleyball class
 *
 * @package LeagueManager
 * @subpackage League_Volleyball
 */
 
 
add_filter( 'leaguemanager_sports', 'leaguemanager_sports_volleyball' );
/**
 * add volleyball to list
 *
 * @param array $sports
 * @return array
 */
function leaguemanager_sports_volleyball( $sports ) {
	$sports['volleyball'] = __( 'Volleyball', 'leaguemanager' );
	return $sports;
}

 
/**
 * Class to implement the League_Volleyball object
 *
 */
final class League_Volleyball extends League {
	/**
	 * sports key
	 *
	 * @var string
	 */
	public $sport = 'volleyball';

	/**
	 * number of sets
	 *
	 * @var int
	 */
	public $num_sets = 5;
	
	
	/**
	 * load custom setings
	 *
	 * @param object $league League object
	 */
	public function __construct($league) {
		parent::__construct($league);
		
		$this->fields_team['sets'] = array('label' => __( 'Sets', 'leaguemanager' ), 'keys' => array('won','lost'));
		$this->fields_team['ballpoints'] = array('label' => __( 'Ballpoints', 'leaguemanager' ), 'keys' => array('plus','minus'));
		
		$this->fields_match['sets'] = array('label' => __( 'Sets', 'leaguemanager' ), 'keys' => array());
		for ( $i = 1; $i <= $this->num_sets; $i++ ) {
			$this->fields_match['sets']['keys'][$i] = array('home', 'away');
		}
		
		add_filter( 'leaguemanager_point_rules_list', array(&$this, 'getPointRuleList') );
		add_filter( 'leaguemanager_point_rules',  array(&$this, 'getPointRules') );
		
		add_filter( 'team_points_'.$this->sport, array(&$this, 'calculatePoints'), 10, 4 );
		
		add_action( 'league_settings_'.$this->sport, array(&$this, 'leagueSettings') );
	}


	/**
	 * get Point Rule list
	 *
	 * @param array $rules
	 * @return array
	 */
	public function getPointRuleList( $rules ) {
		$rules[$this->sport] = __( 'Volleyball', 'leaguemanager' );

		return $rules;
	}
	
	
	/**
	 * get Point rules
	 *
	 * @param array $rules
	 * @return array
	 */
	public function getPointRules( $rules ) {
		$rules[$this->sport] = array( 'forwin' => 3, 'fordraw' => 0, 'forloss' => 0, 'forwin_3_2' => 2, 'forloss_3_2' => 1 );
		
		return $rules;
	}
	
	
	/**
	 * add league settings
	 *
	 * @param object $league
	 */
	public function leagueSettings( $league ) {
		echo "<tr valign='top'>";
		echo "<th scope='row'><label for='num_sets'>".__('Number of Sets', 'leaguemanager')."</label></th>";
		echo "<td><input type='text' name='settings[num_sets]' id='num_sets' value='".$league->num_sets."' size='3' /></td>";
		echo "</tr>";
	}
	
	
	/**
	 * Re-calculate primary points
	 *
	 * @param array $points
	 * @param int $team_id
	 * @param array $rule
	 * @param array $matches
	 * @return array
	 */
	public function calculatePoints( $points, $team_id, $rule, $matches ) {
		extract($rule);

		$num_won_3_2 = $this->getNumWonMatches3_2( $team_id, $matches );
		$num_lost_3_2 = $this->getNumLostMatches3_2( $team_id, $matches );

		$points['plus'] = $points['plus'] - $num_won_3_2 * $forwin + $num_won_3_2 * $forwin_3_2 + $num_lost_3_2 * $forloss_3_2;
		$points['minus'] = $points['minus'] - $num_lost_3_2 * $forwin + $num_won_3_2 * $forloss_3_2 + $num_lost_3_2 * $forwin_3_2;

		return $points;
	}
	
	
	/**
	 * get number of won matches after overtime
	 *
	 * @param int $team_id
	 * @param array $matches
	 * @return int
	 */
	private function getNumWonMatches3_2( $team_id, $matches ) {
		$num = 0;
		foreach ( $matches AS $match ) {
			$custom = maybe_unserialize($match->custom);
			
			if ( $team_id == $match->winner_id ) {
				// Home match won 3:2
				if ( $match->home_team == $team_id && $match->home_points == 3 && $match->away_points == 2 )
					$num++;
				
				// Away match won 3:2
				if ( $match->away_team == $team_id && $match->home_points == 2 && $match->away_points == 3 )
					$num++;
			}
			
		}
		return $num;
	}


	/**
	 * get number of lost matches after overtime
	 *
	 * @param int $team_id
	 * @param array $matches
	 * @return int
	 */
	private function getNumLostMatches3_2( $team_id, $matches ) {

		$num = 0;
		foreach ( $matches AS $match ) {
			$custom = maybe_unserialize($match->custom);
			
			if ( $team_id == $match->loser_id ) {
				// Home match lost 3:2
				if ( $match->away_team == $team_id && $match->home_points == 3 && $match->away_points == 2 )
					$num++;
				
				// Away match lost 3:2
				if ( $match->home_team == $team_id && $match->home_points == 2 && $match->away_points == 3 )
					$num++;
			}
		}
		return $num;
	}
	
	
	/**
	 * rank Teams
	 *
	 * @param array $teams
	 * @return array of teams
	 */
	protected function rankTeams( $teams ) {
		$points = $set_diff = $won_sets = $ballpoints_diff = $ballpoints = array();
		foreach ( $teams AS $key => $row ) {
			$points[$key] = $row->points['plus'];
			$set_diff[$key] = $row->sets['won']-$row->sets['lost'];
			$won_sets[$key] = $row->sets['won'];
			$ballpoints_diff[$key] = $row->ballpoints['plus']-$row->ballpoints['minus'];
			$ballpoints[$key] = $row->ballpoints['plus'];
		}

		array_multisort( $points, SORT_DESC, $set_diff, SORT_DESC, $won_sets, SORT_DESC, $ballpoints_diff, SORT_DESC, $ballpoints, SORT_DESC, $teams );
		return $teams;
	}

	
	/**
	 * get standings data for given team
	 *
	 * @param Team $team
	 * @param array $data
	 * @param array $matches
	 * @return array number of runs for and against as assoziative array
	 */
	protected function getStandingsData( $team, $data = array(), $matches = false ) {
		$data['sets'] = array( "won" => 0, "lost" => 0 );
		$data['ballpoints'] = array( 'plus' => 0, 'minus' => 0 );

		if ( !$matches ) $matches = $this->getMatches( array("team_id" => $team->id, "limit" => false, "cache" => false, "match_day" => -1, "reset_query_args" => true) );
		foreach ( $matches AS $match ) {
			// Home Match
			if ( $team->id == $match->home_team ) {
				$data['sets']['won'] += $match->home_points;
				$data['sets']['lost'] += $match->away_points;
	
				foreach ( $match->sets AS $s => $set ) {
					$data['ballpoints']['plus'] += $set['home'];
					$data['ballpoints']['minus'] += $set['away'];
				}
			} else {
				$data['sets']['won'] += $match->away_points;
				$data['sets']['lost'] += $match->home_points;
	
				foreach ( $match->sets AS $s => $set ) {
					$data['ballpoints']['plus'] += $set['away'];
					$data['ballpoints']['minus'] += $set['home'];
				}
			}
		}
		
		return $data;
	}
	
	
	/**
	 * update match results and automatically calculate apparatus points and score
	 *
	 * @param Match $match
	 * @return Match
	 */
	protected function updateResults( $match ) {
		$force = true;

		if (is_admin() && current_user_can('update_results')) {
			// update primary points
			if ( $force || $match->home_points == "" && $match->away_points == "" ) {
				$score = array( 'home' => 0, 'guest' => 0 );
				for ( $i = 1; $i <= $this->num_sets; $i++ ) {
					if ( $match->sets[$i]['home'] != '' && $match->sets[$i]['away'] != '' ) {
						if ( $match->sets[$i]['home'] > $match->sets[$i]['away'] ) {
							$score['home'] += 1;
						} elseif ($match->sets[$i]['home'] < $match->sets[$i]['away']) {
							$score['guest'] += 1;
						}
					}
				}
				
				$match->home_points = $score['home'];
				$match->away_points = $score['guest'];
			}
		}
		
		return $match;
	}
}
?>